#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <iostream>
#include <cmath>


#include "engine.h"
#include "timer.h"

#define KEY_DELAY   500
#define KEY_REPEAT  80

char key[360];
int  mouse[2];				// Mouse position
int  mouseButton[3];		// Mouse events
bool g_resize;
int  g_window_size[2];


// ============================== RESIZE WINDOW ======================================= //
//Function called when resize event is detected
int resizeWindow( int width, int height )
{
   
    const SDL_VideoInfo* info = SDL_GetVideoInfo();	
	int vidFlags = SDL_OPENGL | SDL_GL_DOUBLEBUFFER | SDL_HWPALETTE | SDL_RESIZABLE ;
	if (info->hw_available) {vidFlags |= SDL_HWSURFACE;}
	else {vidFlags |= SDL_SWSURFACE;}
	int bpp = info->vfmt->BitsPerPixel;
	SDL_SetVideoMode(width, height, bpp, vidFlags);
	
	
	glViewport( 0, 0, width, height );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45, (float)width/height, 0.1, 100 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

    return( true );
}

// ================================== EVENTS  ========================================= //
bool events()	// Process pending events
{
	SDL_Event event;
	if( SDL_PollEvent(&event) )
	{
		switch( event.type )
	{
	//TODO: CHANGE THIS
	case SDL_KEYDOWN 		: key[ event.key.keysym.sym ]=true ;   break;
	case SDL_KEYUP   		: key[ event.key.keysym.sym ]=false;   break;
	case SDL_VIDEORESIZE	: g_window_size[0] = event.resize.w; 
							  g_window_size[1] = event.resize.h; 
							  g_resize = true; break;
	case SDL_MOUSEBUTTONDOWN: mouse[0] = event.button.x; mouse[1]=event.button.y; 
							  mouseButton[event.button.button] = true;			break;
	case SDL_MOUSEBUTTONUP	: mouseButton[event.button.button] = false;			break;
	case SDL_QUIT    		: return false; break;
	}
}
return true;
}


// ========================================================================================= //
// ========================================================================================= //

//============================= ENGINE : CONSTRUCTOR ======================================= //
// Initialize variables
Engine::Engine(int w, int h){

	 m_actualPhoto =0;
	 m_loadedTill = 0;
	 m_texturesLoaded = 0;
	 m_next = false;
	 m_prev = false;
	 m_direct=false;
	 g_resize = false;
	 
	 m_window_size[0] = w;
	 m_window_size[1] = h;
	 m_scroll.set_window_size(w,h);
	
};
Engine::~Engine()
{

	//Launch the end fucntion for each plugin
	for(unsigned i=0; i<m_plugins.size(); i++)
	{
		m_plugins[i].gloobus->end();
	}
};

//============================= ENGINE : INIT ==============================================//
// Read configuration
// Start thread to load images
// Load Plugins
void Engine::init(string dir)
{
	read_config();					// READ THE CONFIGURATION FILE
	m_PManager.load_plugins();		// LOAD ALL PLUGINS
    m_font.init();					// Initialize Font
	get_dir(dir);					// Load Files and creates its plugin

	SDL_EnableKeyRepeat(KEY_DELAY, KEY_REPEAT);	// Enable Key Repeat

	if(m_plugins.size() > 0)
	{
		SDL_CreateMemberThread(this, &Engine::run);	// Thread to load the files
	
		while(m_loadedTill<=0) {}				    // Wait till the fist is loaded		

		m_text = m_plugins[0].gloobus->get_filename();					
		m_text = m_text.substr(m_text.rfind('/',m_text.length())+1);	// Cut just filename
		m_scroll.set_n_files(m_plugins.size());
	}
	else
	{
		printf("There are no previews for this folder");
		exit(-1);
	}
	// ======= SCROLLBAR
	m_scroll.load_images();

};

//============================= ENGINE : UPDATE=============================================//
// Handle events
bool Engine::update(void)
{
	int direction;
	int finalPhoto = -1;
	
	init_plugins();				// Initialize the plugins when they are loaded

	if(g_resize)				// If window resized
	{
		g_resize = false;
		resizeWindow(g_window_size[0],g_window_size[1]);				//Reisze window
		m_scroll.set_window_size(g_window_size[0],g_window_size[1]);	//Set window size for scroll
		m_font.set_window_size(g_window_size[0],g_window_size[1]);		//Set window size for font
	}
	if(events())
	{
		//====QUIT
		if(key[SDLK_q]) { 	
			return false;
		}		
		
		
		//====MOUSE BUTTON DOWN
		if(mouseButton[SDL_BUTTON_LEFT])	
		{
			//printf("Button pressed: %d %d\n",mouse[0],mouse[1]);
			mouseButton[SDL_BUTTON_LEFT] = false;
			
			if(m_scroll.is_right_pressed(mouse[0],mouse[1]) && m_actualPhoto<m_loadedTill-1 ) key[SDLK_RIGHT]=true;
			if(m_scroll.is_left_pressed(mouse[0],mouse[1]) && m_actualPhoto>0)  		  	  key[SDLK_LEFT]=true;

			finalPhoto = m_scroll.is_scrollbar_pressed(mouse[0],mouse[1]);					  //SCROLLBAR
			if(finalPhoto != -1 && finalPhoto !=m_actualPhoto) key[SDLK_t] = true;

			if(!key[SDLK_t])
			{
				finalPhoto = m_scroll.is_photo_pressed(mouse[0],mouse[1]);					 //PHOTO PRESSED
				if(finalPhoto != 0)
				{
					finalPhoto = m_actualPhoto + finalPhoto;
					if(finalPhoto >=0 && finalPhoto <m_loadedTill && finalPhoto != m_actualPhoto)
					{
						key[SDLK_t] = true;
						printf("FinalPhoto: %d\n",finalPhoto);
					}
				}
			}
		}
		
		//====DIRECT POSITION
		if(key[SDLK_t])						
		{	
			
			int dist = m_actualPhoto - finalPhoto;		//To know direction
							
			if(dist!=0)
			{
				direction = dist>0 ? -1 : 1;
				m_actualPhoto=finalPhoto;
				timer::Instance()->setStartTime();
				
				key[SDLK_t] = false;
				m_next = false;				
				m_prev = false;
				m_direct = true;
				
				for(unsigned int i=0; i < m_plugins.size(); i++)		
				{	
					set_direct_position(i,direction);					// SET Covers initial position
				}		
			}
		}

		// === RIGHT
		if(key[SDLK_RIGHT] && m_actualPhoto<m_loadedTill-1 && !m_direct) 
		{ 	
			if(m_next) m_actualPhoto++;			// When is in semi change
			if(m_actualPhoto< m_loadedTill-1)
			{	
				key[SDLK_RIGHT] = false;		// Key repeat
				m_next = true;
				m_prev = false;					
				timer::Instance()->setStartTime();

				for(unsigned int i=0; i < m_plugins.size(); i++)		
				{	
					set_initial_position(i,1);				// SET Covers initial position
				}	
				m_text = m_plugins[m_actualPhoto].gloobus->get_filename();
				m_text = m_text.substr(m_text.rfind('/',m_text.length())+1);	// Cut just filename
			}
			else m_actualPhoto--;
		}
		
		// === LEFT
		if(key[SDLK_LEFT] && m_actualPhoto>0 && !m_direct) 
		{ 	
			if(m_prev) m_actualPhoto--;		// When is in semi change
			if(m_actualPhoto>0)
			{
				key[SDLK_LEFT] = false;			// Key repeat
				m_next = false;				
				m_prev = true;
				timer::Instance()->setStartTime();

				for(unsigned int i=0; i < m_plugins.size(); i++)		
				{	
					set_initial_position(i,-1);						// SET Covers initial position
				}	
				m_text = m_plugins[m_actualPhoto].gloobus->get_filename();
				m_text = m_text.substr(m_text.rfind('/',m_text.length())+1);	// Cut just filename
			}
			else m_actualPhoto++;
		}

		//===== LAUNCH PREVIEW
		if(key[SDLK_SPACE] && !m_next && !m_prev)			// LAUNCH PREVIEW!!
		{ 	
			string cmd;
			cmd="/usr/share/gloobus/preview \""+ m_plugins[m_actualPhoto].gloobus->get_filename() +"\"";
			//printf("CMD: %s\n",cmd.c_str());
			system(cmd.c_str());	
		
		}
		// ===================================================================== //
		if(m_next || m_prev || m_direct)
		{
			//COVERFLOW
			double time = timer::Instance()->obtainTime();
			

			if(!m_direct) direction = m_next ? 1:-1;			// When no direct position

			for(unsigned int i=0; i< m_plugins.size(); i++)
			{
				if(m_direct)	update_direct_photo(i,time,direction);
				else		 	update_photo(i,time,direction); 
			}

			int t;
			if(!m_direct) t = m_config.cHANGE_TIME;
			else		  t = m_directChangeTime;		
			if( time > t) 
			{ 	
				if(!m_direct) m_actualPhoto+=direction;
				m_next=m_prev=m_direct=false;		

				m_text = m_plugins[m_actualPhoto].gloobus->get_filename();
				m_text = m_text.substr(m_text.rfind('/',m_text.length())+1);	// Cut just filename

				//TODO: Just 10 photos on memory
			}
		}
	}
	else {return(false);}

	return true;
};
//============================= ENGINE : SET INITIAL POS ====================================//
// Depending of the photo number and actual photo it calculates the initial and final pos of the transition
// This is just for next and prev for direct position use the set_direct_position function
void Engine::set_initial_position(int i, int direction)
{
	int dist = i - m_actualPhoto - direction;

	m_plugins[i].initialPos = m_plugins[i].xPos;
	
	if(i==m_actualPhoto)				 m_plugins[i].finalPos = m_config.cOVER_SPACE;
	else if(i==m_actualPhoto+direction)  m_plugins[i].finalPos = 0;
	else {
			if(dist>0)	{ m_plugins[i].finalPos = (m_config.cOVER_SPACE + m_config.sPACE*(dist));}
			else		{ m_plugins[i].finalPos = (-m_config.cOVER_SPACE + m_config.sPACE*(dist));}
		 }

}

//============================= ENGINE : UPDATE PHOTO =======================================//
void Engine::update_photo(int i,double time,int direction)
{
	//double x;
	double x2;
	double inici,fi;
	
	// y = - (final_pos - initial_pos / change_time²) (x - change_time)² + final_dist
	
	inici = m_plugins[i].initialPos;
	fi = m_plugins[i].finalPos;

	if(i==m_actualPhoto)							// Actual photo, we move from 0 to the next 
	{
			//ANGLE
			//x = m_config.cOVER_ANGLE / sqrt(m_config.cHANGE_TIME);		
			//m_plugins[i].angle=direction*sqrt(time)*x;
			
			x2 = (double)(m_config.cOVER_ANGLE) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);			// x2 function
			m_plugins[i].angle = direction* (-x2 * (time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME) 
			+ m_config.cOVER_ANGLE);
			
			//printf("Time: %f, X2: %f - ANGLE: %f \n",time,x2,m_plugins[i].angle);
		
			//XPOS
			//if(direction==1) x = (inici + fi) / sqrt(m_config.cHANGE_TIME);
			//else			 x = (fi-inici) / sqrt(m_config.cHANGE_TIME);	
			//m_plugins[i].xPos = -direction*sqrt(time)*x + inici;
			
			if(direction==1) x2 = (double)(inici + fi) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);// x2 function
			else			 x2 = (double)(fi - inici) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);// x2 function			
				
			m_plugins[i].xPos = -direction*(-x2*(time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME)+ fi);


			//ZPOS	
			//x = (double)m_config.zPOSITION / sqrt(m_config.cHANGE_TIME);			
			//m_plugins[i].zPos= m_config.zPOSITION - (x*sqrt(time));
			
			x2 = (double)(0 - m_config.zPOSITION) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);
			m_plugins[i].zPos = (-x2*(time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME));
					
	}
	else 											// Photo to be on front
	{
	
		if(i==m_actualPhoto+direction)
		{
				//ANGLE
				//x = m_config.cOVER_ANGLE / sqrt(m_config.cHANGE_TIME);
				//m_plugins[i].angle=direction* (sqrt(time)*x - m_config.cOVER_ANGLE);
				x2 = (double)(m_config.cOVER_ANGLE) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);			// x2 function
				m_plugins[i].angle=direction* (-x2 * (time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME) 
				);
				
				
				//XPOS
				//x = (fi-inici) / sqrt(m_config.cHANGE_TIME);
				//m_plugins[i].xPos = sqrt(time)*x + inici;
				x2 = (double)(fi-inici) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);			// x2 function
				m_plugins[i].xPos = (-x2*(time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME)+ fi);
				

				//ZPOS
				//x = (double)m_config.zPOSITION / sqrt(m_config.cHANGE_TIME);	
				//m_plugins[i].zPos= (x*sqrt(time));
				x2 = (double)(0 + m_config.zPOSITION) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);
				m_plugins[i].zPos = (-x2*(time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME) + m_config.zPOSITION);
		}
		else										// Other Photos
		{
			//ANGLE
			if(i-m_actualPhoto > 0)			m_plugins[i].angle = -m_config.cOVER_ANGLE;
			else							m_plugins[i].angle =  m_config.cOVER_ANGLE;
			//xPOS
			//x = (fi-inici) / sqrt(m_config.cHANGE_TIME);	
			//m_plugins[i].xPos = sqrt(time)*x + inici;
			x2 = (double)(fi-inici) / (m_config.cHANGE_TIME*m_config.cHANGE_TIME);			// x2 function
			m_plugins[i].xPos = (-x2*(time - m_config.cHANGE_TIME)*(time - m_config.cHANGE_TIME)+ fi);
			
			m_plugins[i].zPos= 0;
		}
	} 
}
//============================= ENGINE : SET DIRECT POS ====================================//
//It calculates the initial and final position of the photo, but this time using direct position
//for next and prev function use the set_initial_position function
void Engine::set_direct_position(int i, int direction)
{
	int dist;
	if(direction == -1)	dist = i - m_actualPhoto - direction;
	else				dist = i - m_actualPhoto;

	m_directChangeTime = m_config.cHANGE_TIME + (abs(dist)-1)*10;		// Change time depengin on how distant it is
	
	m_plugins[i].initialPos = m_plugins[i].xPos;
	
	if(i==m_actualPhoto)				 m_plugins[i].finalPos = 0;
	else if(i==m_actualPhoto-direction)  m_plugins[i].finalPos = m_config.cOVER_SPACE;
	else {
			if(dist>0)	{ m_plugins[i].finalPos = (m_config.cOVER_SPACE + m_config.sPACE*(dist));}
			else		{ m_plugins[i].finalPos = (-m_config.cOVER_SPACE + m_config.sPACE*(dist));}
		 }

}
//============================= ENGINE : UPDATE DIRECT PHOTO =======================================//
void Engine::update_direct_photo(int i,double time,int direction)
{
	//double x;
	double x2;
	double inici,fi;

	inici = m_plugins[i].initialPos;
	fi = m_plugins[i].finalPos;

	if(i==m_actualPhoto - direction)				// Actual photo, we move from 0 to the next 
	{
			//ANGLE
			//x = m_config.cOVER_ANGLE / sqrt(m_directChangeTime);		
			//m_plugins[i].angle=direction*sqrt(time)*x;
			x2 = (double)(m_config.cOVER_ANGLE) / (m_directChangeTime*m_directChangeTime);			// x2 function
			m_plugins[i].angle = direction* (-x2 * (time - m_directChangeTime)*(time - m_directChangeTime) 
			+ m_config.cOVER_ANGLE);
		
			//XPOS
			//if(direction==1) x = (inici + fi) / sqrt(m_directChangeTime);
			//else			   x = (fi-inici) / sqrt(m_directChangeTime);			
			//m_plugins[i].xPos = -direction*sqrt(time)*x + inici;
			
			if(direction==1) x2 = (double)(inici + fi) / (m_directChangeTime*m_directChangeTime);// x2 function
			else			 x2 = (double)(fi - inici) / (m_directChangeTime*m_directChangeTime);// x2 function			
				
			m_plugins[i].xPos = -direction*(-x2*(time - m_directChangeTime)*(time - m_directChangeTime)+ fi);


			//ZPOS	
			//x = (double)m_config.cOVER_SPACE / sqrt(m_directChangeTime);			
			//m_plugins[i].zPos= m_config.zPOSITION - (x*sqrt(time));
			
			x2 = (double)(0 - m_config.zPOSITION) / (m_directChangeTime*m_directChangeTime);
			m_plugins[i].zPos = (-x2*(time - m_directChangeTime)*(time - m_directChangeTime));
			//printf("ZPOS: %f\n",m_photos[i].zPos);
	}
	else 											// Photo to be on front
	{
	
		if(i==m_actualPhoto)
		{
				//ANGLE
				//x = m_config.cOVER_ANGLE / sqrt(m_directChangeTime);
				//m_plugins[i].angle=direction* (sqrt(time)*x - m_config.cOVER_ANGLE);
				
				x2 = (double)(m_config.cOVER_ANGLE) / (m_directChangeTime*m_directChangeTime);			// x2 function
				m_plugins[i].angle = direction* (-x2 * (time - m_directChangeTime)*(time - m_directChangeTime));

				//XPOS
				//x = (fi-inici) / sqrt(m_directChangeTime);
				//m_plugins[i].xPos = sqrt(time)*x + inici;
				x2 = (double)(fi-inici) / (m_directChangeTime*m_directChangeTime);			// x2 function
				m_plugins[i].xPos = (-x2*(time - m_directChangeTime)*(time - m_directChangeTime)+ fi);

				//ZPOS
				//x = (double)m_config.zPOSITION / sqrt(m_directChangeTime);	
				//m_plugins[i].zPos= (x*sqrt(time));
				x2 = (double)(0 + m_config.zPOSITION) / (m_directChangeTime*m_directChangeTime);
				m_plugins[i].zPos = (-x2*(time - m_directChangeTime)*(time - m_directChangeTime) + m_config.zPOSITION);
				//printf("ZPOS: %f\n",m_photos[i].zPos);
		}
		else										// Other Photos
		{
			//ANGLE
			if(i-m_actualPhoto > 0)			m_plugins[i].angle = -m_config.cOVER_ANGLE;
			else							m_plugins[i].angle =  m_config.cOVER_ANGLE;
			//xPOS
			//x = (fi-inici) / sqrt(m_directChangeTime);	
			//m_plugins[i].xPos = sqrt(time)*x + inici;
			x2 = (double)(fi-inici) / (m_directChangeTime*m_directChangeTime);			// x2 function
			m_plugins[i].xPos = (-x2*(time - m_directChangeTime)*(time - m_directChangeTime)+ fi);
			m_plugins[i].zPos= 0;
		}
	} 
}
//===========================================================================================//
//==							ENGINE : RENDER											   ==//
//===========================================================================================//
void Engine::render(void)
{
	int i;
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear previous draw
																// MOVE EVERYTHING A little UP
		// FOR TRANSPARENCY, NEED TO LOAD FROM 0 TO Actual-1 then from LAST TO Actual+1	and finaly Actual
		// PHOTO ==========================================
		for(int j=0; j< m_loadedTill; j++)
		{
		i = j;
		if(m_actualPhoto - j > 0) 	 	i = j;
		if(m_actualPhoto - j == 0)		i = m_loadedTill - 1;
		if(m_actualPhoto - j < 0)		i = m_loadedTill + m_actualPhoto - j - 1;
		
		glLoadIdentity();
		
		glRotatef(m_config.uP_ANGLE,1,0,0);			// UP VIEW ANGLE	
		glTranslatef(0,-float(m_config.uP_ANGLE)/float(20),0);						// UP VIEW ANGLE
		
		glTranslatef(0,0.1, -5.3);								//We get some far away and up
		
		glTranslatef(m_plugins[i].xPos,0,m_plugins[i].zPos);	// Moving
		glRotatef(m_plugins[i].angle, 0, 1, 0);					// 3D Effect rotation

		
		// REFLECTION	
		glPushMatrix();
			glScalef(1, -1, 1);  				// INVERT
			glTranslatef(0, 2, 0); 				// MOVE DOWN REFLECTION

				if(m_plugins[i].fadeIn<1)
				{
    				glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);			// SET FADE IN
	   				glColor4f(1.0f, 1.0f, 1.0f, m_plugins[i].fadeIn-0.5f);
				}
				else
				{
					glBlendFunc(GL_ONE,GL_ZERO);								// SET REFLECTION
					glColor4f(0.5f,0.5f,0.5f,m_plugins[i].fadeIn-0.5f);
				}
				m_plugins[i].gloobus->render();
		glPopMatrix();
		
		do_fade_in(i);
		m_plugins[i].gloobus->render();				// DRAW EACH PHOTO
		
		}
		
		// TEXT  ==========================================
		render_text();			// Render the current file text
		
		// SCROLLBAR ======================================
		m_scroll.render();
		
		// DRAW EVERYTHING ================================	
		glFlush();									// Better performance
		SDL_GL_SwapBuffers();						// DRAW SCENE
};
// ============================ ENGINE : DO FADE IN =========================================//
void Engine::do_fade_in(int i)
{
		//TODO: FADE_IN with timer.h
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);		
		glColor4f(1.0f, 1.0f, 1.0f, m_plugins[i].fadeIn);

		if(m_plugins[i].fadeIn < 1)
		{
			double time = timer::Instance()->time_since(m_plugins[i].startTime);
			m_plugins[i].fadeIn = time/500;		
		}
}
//============================= ENGINE : Render Text =======================================//
void Engine::render_text()
{
		glPushMatrix();
		glBlendFunc(GL_ONE, GL_ONE);
			m_font.glEnable2D();
					m_font.SDL_GL_RenderText(m_text,&m_font.m_position);
			m_font.glDisable2D();
		glPopMatrix();
}


//============================= ENGINE : DRAW =================================//
bool Engine::draw(void)
{
	if(m_next || m_prev) return true;
	else return false;
}

//============================= ENGINE : THREAD =================================//
int Engine::run(void)
{
	printf("\t\tThread\n");					

	for (unsigned int i = 0;i < m_plugins.size();i++) 
	{
			m_plugins[i].gloobus->load();
	
			// Position on CoverFlow
			if(i==0)			//FIRST PHOTO
			{
				m_plugins[i].angle = 0;
				m_plugins[i].xPos = 0;
				m_plugins[i].zPos = m_config.zPOSITION;
				m_plugins[i].fadeIn = 0;			// All files appears from nowere
			}
			else	 			//OTHER PHOTOS
			{				
				m_plugins[i].angle = -m_config.cOVER_ANGLE;
				m_plugins[i].xPos = m_config.cOVER_SPACE + (i-1)*m_config.sPACE;
				m_plugins[i].zPos = 0;
				m_plugins[i].fadeIn = 0;			// All files appears from nowere
			}
			m_plugins[i].startTime = timer::Instance()->get_actual_time();		
			m_loadedTill++;		
	}
	printf("\t\tEND Thread\n");
	return 0;
}

//============================= ENGINE : INIT PLUGINS =================================//
void Engine::init_plugins()
{
	for(int i=m_texturesLoaded;i<m_loadedTill;i++)
	{
			m_plugins[i].gloobus->init();
			m_texturesLoaded++;
	}

}
// ============================= READ DIRECTORY (And create plugins) ================================= //
int Engine::get_dir(string dir)
{

	static unsigned int BUF_SIZE = 1024;
	FILE *dl;					// File Handle
	char in_buf[BUF_SIZE]; 		// input buffer for lib
    char name[BUF_SIZE];
    string cmd = "ls \"" + dir+ "\"  -1 -X";

	// == Read Directory == //
 	dl = popen(cmd.c_str(), "r");		
   	if(!dl){      
		perror("popen");      
		exit(-1);   	
	}
	
	// Test each filename and create its plugin
	while(fgets(in_buf, BUF_SIZE, dl))
	{
		char *ws = strpbrk(in_buf, "\n");	// trim off the whitespace
		if(ws) *ws = '\0';
		sprintf(name, "%s/%s",dir.c_str(), in_buf);	      	// append ./ to the front of the filename
		printf("FILE: %s\n",name);		
		
		string key = m_PManager.search_plugin_for_file(name);	// Search the plugin to use for this file
		if(key == "NULL"){ 
			printf("This filetype has no preview function\n");
			
		}
		else
		{	
			// Create iGloobus for the file
			plugins_T plugin;
			plugin.gloobus = factory[key]();				// Generate Plugin
			plugin.gloobus->set_filename(name);				// Set filename
			plugin.gloobus->set_gloobus_type(GLOOBUS);		// Set as Gloobus
					
			m_plugins.push_back(plugin);
		}		
	}
	fclose(dl);

	return 0;
}
// ============================= READ CONFIG  =============================================== //
// Load config variables on file or default ones
void Engine::read_config()
{
	ConfigFile config("/usr/share/gloobus/gloobus.config");
	m_config.cOVER_ANGLE =  config.read(	"COVER_ANGLE",			70	);
	m_config.cOVER_SPACE =  config.read(	"COVER_SPACE",			2.3 );
	m_config.sPACE 		 =  config.read(	"SPACE",				0.5	);
	m_config.cHANGE_TIME =  config.read(	"CHANGE_TIME",			300 );
	m_config.zPOSITION   =  config.read(	"ZPOSITION",			2.3 );
	m_config.uP_ANGLE    =  config.read(	"UP_ANGLE",				0 );

	printf("COVER ANGLE: %d\n",m_config.cOVER_ANGLE);
}

