#include "font.h"
#include "ConfigFile.h"

//============================= FONT : INIT =================================//
int Font2::init()
{
	TTF_Init();
	
	if(!(m_font = TTF_OpenFont("/usr/share/gloobus/font.ttf",20))) {
			printf("Error loading font: %s\n", TTF_GetError());
	}

	m_color.r = 255;
	m_color.g = 255;
	m_color.b = 255;

	ConfigFile config("/usr/share/gloobus/gloobus.config");

	m_position.x = 	config.read("WIDTH",1000 )/2;
	m_position.y = 35;
	
	//printf("Initializing font... \n");
	return 0;
}
//============================= FONT : SET WIDTH =================================//
void Font2::set_window_size(int w,int h)
{
	m_position.x = w/2;
}
//============================= FONT : ENABLE 2D =================================//
void Font2::glEnable2D()
{
	int vPort[4];
  
	glGetIntegerv(GL_VIEWPORT, vPort);
  
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
  
	glOrtho(0, vPort[2], 0, vPort[3], -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}
//============================= FONT : DISABLE 2D =================================//
void Font2::glDisable2D()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();   
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();	
}

//============================= FONT : RENDER GL TEXT =================================//
void Font2::SDL_GL_RenderText(string text, SDL_Rect *location)
{
	//TODO: Create just one time the texture!!!
	SDL_Surface *initial;
	SDL_Surface *intermediary;
	int w,h;
	GLuint texture;

	// Use SDL_TTF to render our text 
	initial = TTF_RenderText_Blended(m_font, text.c_str() , m_color);
	
	w = initial->w;
	h = initial->h;
	
	intermediary = SDL_CreateRGBSurface(0, w, h, 32,0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000);

	SDL_BlitSurface(initial, 0, intermediary, 0);
	
	// Tell GL about our new texture 
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_RECTANGLE_ARB, texture);
	glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, 4, w, h, 0, GL_BGRA, GL_UNSIGNED_BYTE, intermediary->pixels );
	
	// GL_NEAREST looks horrible, if scaled... 
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	

	// prepare to render our texture 
	glEnable(GL_TEXTURE_RECTANGLE_ARB);
	glBindTexture(GL_TEXTURE_RECTANGLE_ARB, texture);
	glColor3f(1.0f, 1.0f, 1.0f);
	
	// Draw a quad at location 
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, h);		glVertex2f(location->x - w/2  ,location->y);
		glTexCoord2f(w, h);		glVertex2f(location->x + w/2  ,location->y);
		glTexCoord2f(w, 0.0f);		glVertex2f(location->x + w/2  ,location->y + h);
		glTexCoord2f(0.0f, 0.0f);		glVertex2f(location->x -w/2   ,location->y + h);
	glEnd();
			
	// Clean up 
	SDL_FreeSurface(initial);
	SDL_FreeSurface(intermediary);
	glDeleteTextures(1, &texture);
}

