#include "scrollbar.h"	

//============================= SCROLLBAR : CONSTRUCTOR =============================================//
Scrollbar::Scrollbar(){};
Scrollbar::~Scrollbar(){};

//============================= SCROLLBAR : SET WINDOW SIZE =========================================//
void Scrollbar::set_window_size(int w, int h)
{	
	m_w = w;
	m_h = h;
}
//============================= SCROLLBAR : SET N FILES =============================================//
void Scrollbar::set_n_files(int n)
{
	m_nfiles = n;
	m_margin = 250;
	m_bottomMargin = 20;
	m_arrowSize = 25;
}
//============================= SCROLLBAR : LEFT ARROW PRESSED ? ====================================//
bool Scrollbar::is_left_pressed(int x, int y)
{ 
	//printf("y:%d - H:%d\n",y,m_h);
	if(x > m_margin  			&&  x < m_margin + m_arrowSize &&
	   y < m_h - m_bottomMargin &&  y > m_h - (m_bottomMargin + m_arrowSize)  )   
	{
		printf("Left Arrow Pressed\n"); 
		return true;
	}
	   		
	return false;
};
//============================= SCROLLBAR : RIGHT ARROW PRESSED ?  =================================//
bool Scrollbar::is_right_pressed(int x, int y)
{
	if(x > m_w - m_margin   		&&  x <   m_w - (m_margin - m_arrowSize)   &&
	   y < m_h - m_bottomMargin &&  y > m_h - (m_bottomMargin + m_arrowSize)  )   
	{
		printf("Right Arrow Pressed\n"); 
		return true;
	}
		
	return false;
}
//============================= SCROLLBAR : SCROLLBAR PRESSED ===========================================//		
int Scrollbar::is_scrollbar_pressed(int x, int y)
{
	if(x> m_margin + m_arrowSize && x < m_w - (m_margin)&&
	   y < m_h - m_bottomMargin &&  y > m_h - (m_bottomMargin + m_arrowSize)  )
	{
		int scrollbar_size = m_w - (m_margin+m_arrowSize)*2;		//Widht less 2 margins
		//printf("SC SIZE:%d - Files:%d, X:%d\n",scrollbar_size,m_nfiles,x);
		
		int x1 = (x-(m_margin+m_arrowSize)) * (m_nfiles-1);			//Less left margin
		x1 =  x1 / scrollbar_size;
		
		if(x1 >= m_nfiles) return m_nfiles - 1;	// Saves a segmentation fault 
		else			   return x1;		
		
	}
	else
		return -1;		// No photo pressed
}
//============================= SCROLLBAR : PHOTO PRESSED ===========================================//		
int Scrollbar::is_photo_pressed(int x, int y)
{

	if(y > 80  && y< 300 )			//Test if inside the coverflow canvas
	{
		int cover_width = 140;		//Space from front cover
		int cover_space = 55;		//Space between covers
		int cover;

		int pos = x - m_w / 2;  



		if (abs(pos) < cover_width) cover = 0;
		else				   		cover = (abs(pos) - cover_width) / cover_space;		

		printf("TEST1: %d \n", abs(pos) - cover_width);
		printf("X:%d - POS:%d -- COVER: %d\n",x,pos, cover);
		if(pos > 0 )	return cover;
		else 			return -cover;
	}
	else
		return 0;
}
//============================= SCROLLBAR : INIT ====================================================//		
void Scrollbar::init()
{

}
//============================= SCROLLBAR : LOAD IMAGES =============================================//	
void Scrollbar::load_images()
{
	m_leftArrow = NULL;
	m_leftArrow = IMG_Load("/usr/share/gloobus/left_arrow.png");

	m_rightArrow = NULL;
	m_rightArrow = IMG_Load("/usr/share/gloobus/right_arrow.png");

	if(m_leftArrow == NULL || m_rightArrow == NULL) {
			printf("Error loading left or right arrow.png\n"); exit(-1); 
	}

	// == LOAD TEXTURE == //
	glGenTextures(1, &m_left_textureID);
	glBindTexture(GL_TEXTURE_RECTANGLE_ARB,m_left_textureID);
	glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA, m_leftArrow->w, m_leftArrow->h, 0,
			       GL_RGBA, GL_UNSIGNED_BYTE, m_leftArrow->pixels );

	glGenTextures(1, &m_right_textureID);
	glBindTexture(GL_TEXTURE_RECTANGLE_ARB,m_right_textureID);
	glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA, m_rightArrow->w,m_rightArrow->h, 0,
			       GL_RGBA, GL_UNSIGNED_BYTE, m_rightArrow->pixels );
			       

}
//============================= SCROLLBAR : RENDER ==================================================//	
void Scrollbar::render()
{
	glEnable2D();
		draw_arrows();
	glDisable2D();
}
//============================= SCROLLBAR : RENDER ARROWS ===========================================//		
void Scrollbar::draw_arrows()
{
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);		// Transparent background
	glEnable(GL_TEXTURE_RECTANGLE_ARB);
	glBindTexture( GL_TEXTURE_RECTANGLE_ARB, m_left_textureID );
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	// == LEFT ARROW
	glBegin(GL_QUADS);
			glTexCoord2f(0.0f,  m_leftArrow->h); 			glVertex2f( m_margin, m_bottomMargin);	
			glTexCoord2f(m_leftArrow->w,  m_leftArrow->h); 	glVertex2f( m_margin + m_arrowSize,  m_bottomMargin);
			glTexCoord2f(m_leftArrow->w, 0); 				glVertex2f( m_margin + m_arrowSize,  m_bottomMargin + m_arrowSize);
			glTexCoord2f(0.0f, 0);							glVertex2f( m_margin, m_bottomMargin + m_arrowSize);
	glEnd();
	
	// == RIGHT ARROW
	glBindTexture( GL_TEXTURE_RECTANGLE_ARB, m_right_textureID );
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glBegin(GL_QUADS);
			glTexCoord2f(0.0f, m_leftArrow->h); 			glVertex2f( m_w - m_margin, m_bottomMargin);	
			glTexCoord2f(m_leftArrow->w, m_leftArrow->h); 	glVertex2f( m_w -m_margin + m_arrowSize,  m_bottomMargin);
			glTexCoord2f(m_leftArrow->w, 0); 				glVertex2f( m_w - m_margin + m_arrowSize,  m_bottomMargin + m_arrowSize);
			glTexCoord2f(0.0f, 0);					glVertex2f( m_w -m_margin, m_bottomMargin + m_arrowSize);
	glEnd();
	
	glDisable(GL_TEXTURE_RECTANGLE_ARB);
	
};
void Scrollbar::draw_scroll(){};

//============================= PREVIEW : ENABLE 2D =================================//
void Scrollbar::glEnable2D()
{
	int vPort[4];
  
	glGetIntegerv(GL_VIEWPORT, vPort);
  
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
  
	glOrtho(0, vPort[2], 0, vPort[3], -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}
//============================= PREVIEW : DISABLE 2D =================================//
void Scrollbar::glDisable2D()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();   
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();	
}
