#include "preview.h"

char key[256];				// Key events
int  mouse[2];				// Mouse position
int  mouseButton[3];		// Mouse events

// =========================== CENTER WINDOW ================================================= //
void gtv_center_window(SDL_Surface *screen)
{
    SDL_SysWMinfo info;

    SDL_VERSION(&info.version);
    if ( SDL_GetWMInfo(&info) > 0 ) {
        int x, y;
        int w, h;

        if ( info.subsystem == SDL_SYSWM_X11 ) {
            info.info.x11.lock_func();
            w = DisplayWidth(info.info.x11.display,
                             DefaultScreen(info.info.x11.display));
            h = DisplayHeight(info.info.x11.display,
                             DefaultScreen(info.info.x11.display));
            x = (w - screen->w)/2;
            y = (h - screen->h)/2;
            XMoveWindow(info.info.x11.display, info.info.x11.wmwindow, x, y);
            info.info.x11.unlock_func();
        }

    }
}

// ============================== RESIZE WINDOW ======================================= //
int resizeWindow( int width, int height, bool fullscreen )
{
    const SDL_VideoInfo* info = SDL_GetVideoInfo();	
	int vidFlags = SDL_OPENGL | SDL_GL_DOUBLEBUFFER | SDL_HWPALETTE | SDL_RESIZABLE | SDL_NOFRAME ;
	if (info->hw_available) {vidFlags |= SDL_HWSURFACE;}
	else {vidFlags |= SDL_SWSURFACE;}
	if (fullscreen) { vidFlags |= SDL_FULLSCREEN; width++; height++;}
	
	int bpp = info->vfmt->BitsPerPixel;
	SDL_Surface * screen = SDL_SetVideoMode(width, height, 32, vidFlags);		// Resize window
	
	gtv_center_window(screen);													// Center windows
	
	glViewport( 0, 0, width, height );											// Gl reset
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45, (float)width/height, 0.1, 100 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

    return( true );
}

// ================================== EVENTS  ========================================= //
bool events()	// Process pending events
{
	SDL_Event event;
	if( SDL_PollEvent(&event) )
	{
		switch( event.type )
	{
	case SDL_KEYDOWN : key[ event.key.keysym.sym ]=true ;   break;
	case SDL_KEYUP   : key[ event.key.keysym.sym ]=false;   break;
	case SDL_MOUSEBUTTONDOWN: mouse[0] = event.button.x; mouse[1]=event.button.y; 
							  mouseButton[event.button.button] = true;			break;
	case SDL_MOUSEBUTTONUP	: mouseButton[event.button.button] = false;			break;
	case SDL_VIDEORESIZE: resizeWindow(event.resize.w, event.resize.h, false);  break;
	case SDL_QUIT    : return false; break;
	}
}
return true;
}

// ===================================================================================== //
// == 						PREVIEW CLASS		             						  == //
// ===================================================================================== //

//============================= PREVIEW : INIT ========================================= //
void Preview::init()
{
	m_fullScreen = false;
	load_fullscreen_button();
}

//============================= PREVIEW : UPDATE ========================================= //
bool Preview::update(iGloobus *gloobus)
{
		
	if(events())
	{
		if(key[SDLK_SPACE]) 						//========= QUIT ON PRESS SPACE
		{ 	
			return false;
		}
		if(key[SDLK_f])								//========= PLUGIN UPDATE
		{
			toogle_fullscreen(gloobus);
			key[SDLK_f] = false;
		}

		if(mouseButton[SDL_BUTTON_LEFT])			//======== FULLSCREEN PRESSED
		{
			//printf("Button pressed: %d %d\n",mouse[0],mouse[1]);
			mouseButton[SDL_BUTTON_LEFT] = false;
			if(m_fullScreen) 
			{ 
				if(mouse[0] > m_fullscreenW/2 - 50/2 && mouse[0] < m_fullscreenW/2 + 50/2
				&& mouse[1] < m_fullscreenH - 20      && mouse[1] > m_fullscreenH - 10 - 100)
				toogle_fullscreen(gloobus);				
			}
			else
			{
				if(mouse[0] > m_width/2 - 50/2 && mouse[0] < m_width/2 + 50/2
				&& mouse[1] < m_height -10     && mouse[1] > m_height - 10 - 50)
				toogle_fullscreen(gloobus);	
			}
		}
		
		
		
		gloobus->update(key);						//========= PLUGIN UPDATE
	}
	else return(false);
	return true;
};

//============================= PREVIEW : RENDER   ======================================= //
void Preview::render(iGloobus *gloobus)
{

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear previous draw

		glLoadIdentity();
		glTranslatef(0,0,-2.4);								// We get some far away
		gloobus->render();									// DRAW THE PLUGIN RENDER
		
		glEnable2D();
			draw_extras();										// Draw fullscreen button
		glDisable2D();

		SDL_GL_SwapBuffers();	
}

//============================= PREVIEW : DRAW EXTRAS ==================================== //
void Preview::draw_extras()
{
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);		// Transparent background
	glEnable(GL_TEXTURE_RECTANGLE_ARB);
	glBindTexture( GL_TEXTURE_RECTANGLE_ARB, m_textureID );
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	//QUAD
	int initW,initH = 10;
	if(m_fullScreen) { initW = m_fullscreenW / 2 - 20;}
	else			 { initW = m_width / 2 - 20; }
	
	glBegin(GL_QUADS);
			glTexCoord2f(0.0f,  m_fullscreen->h); 						glVertex2f( initW, 		initH);	
			glTexCoord2f(m_fullscreen->w, m_fullscreen->h); 			glVertex2f( initW + 40, initH);
			glTexCoord2f(m_fullscreen->w, 0); 							glVertex2f( initW + 40, initH + 40);
			glTexCoord2f(0.0f, 0);										glVertex2f( initW, 		initH + 40);
	glEnd();
	glDisable(GL_TEXTURE_RECTANGLE_ARB);
}

//============================= PREVIEW : LOAD FULL SCREN IMAGE ========================= //
void Preview::load_fullscreen_button()
{
	m_fullscreen = NULL;
	m_fullscreen = IMG_Load("/usr/share/gloobus/fullscreen.png");

	if(m_fullscreen == NULL) {
			printf("Error loading fullscreen.png\n"); exit(-1); 
	}

	// == LOAD TEXTURE == //
	glGenTextures(1, &m_textureID);
	glBindTexture(GL_TEXTURE_RECTANGLE_ARB,m_textureID);
	glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA, m_fullscreen->w, m_fullscreen->h, 0,
			       GL_RGBA, GL_UNSIGNED_BYTE, m_fullscreen->pixels );

}
// ============================= PREVIEW: TOOGLE SCREEN ============================ //
void Preview::toogle_fullscreen(iGloobus *gloobus)
{
	if(!m_fullScreen)
	{
		printf("Fullscreen mode\n");
		m_fullScreen = true;
		/*for(int i=m_width; i<m_fullscreenW; i+=100){
			resizeWindow(i,m_fullscreenH -1, false);
			render(gloobus);
		}*/
		resizeWindow(m_fullscreenW -1, m_fullscreenH-1,true);
		render(gloobus);
	}
	else
	{
		printf("No Fullscreen mode\n");
		m_fullScreen = false;
		resizeWindow(m_width,m_height,false);
		render(gloobus);

	}
}
// =========================== PREVIEW: SET SIZE ==================================== //
void Preview::set_size(int w, int h)
{
	m_width = w;
	m_height = h;
	
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	if ( SDL_GetWMInfo(&info) > 0 ) {

	    if ( info.subsystem == SDL_SYSWM_X11 ) {
	        info.info.x11.lock_func();
	        m_fullscreenW = DisplayWidth(info.info.x11.display, DefaultScreen(info.info.x11.display));
	        m_fullscreenH = DisplayHeight(info.info.x11.display,DefaultScreen(info.info.x11.display));
	      	info.info.x11.unlock_func();

		    }
		}
	printf("RATIO: %f\n",(float)m_width/(float)m_height);
}

//============================= PREVIEW : ENABLE 2D =================================//
void Preview::glEnable2D()
{
	int vPort[4];
  
	glGetIntegerv(GL_VIEWPORT, vPort);
  
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
  
	glOrtho(0, vPort[2], 0, vPort[3], -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}
//============================= PREVIEW : DISABLE 2D =================================//
void Preview::glDisable2D()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();   
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();	
}
